"use strict";
/* 
 * 17 -> Ctrl
 * 65 -> A
 * 69 -> E
 * 87 -> W
 * 85 -> U
 * 84 -> T
 * 8 -> Backspace
 */

function init() {

    var ctrl = false, tarj = null;

    function findlinestart(txt, pos) {
        for (var i = pos - 1; i > 0; i -= 1) {
            if (txt.charAt(i) === '\n') {
                i += 1; // Line starts after the \n!
                break;
            }
        }
        return i;
    }

    function kdown(e) {
        var ls = null, dest = null, end = null, i = null, v = null, c = null, s = null, ch = null;
        e.stopPropagation();
				// Prevent Backspace from acting as back button (broken?).
        if (e.keyCode === 17 || (tarj === null && e.keyCode === 8)) { 
            ctrl = true;
            e.preventDefault();
            return false;
        }
        else if (ctrl && e.keyCode === 65) { // A
            if (tarj !== null) {
                if (tarj.nodeName.toUpperCase() === 'TEXTAREA') {
                    ls = findlinestart(tarj.value, tarj.selectionStart);
                    tarj.setSelectionRange(ls, ls);
                }
                else {
                    tarj.setSelectionRange(0, 0);
                }
                e.preventDefault();
                return false;
            }
            else {
                return true;
            }
        }
        else if (ctrl && e.keyCode === 69) { // E
            if (tarj !== null) {
                dest = tarj.value.length;
                if (tarj.nodeName.toUpperCase() === 'TEXTAREA') {
                    end = tarj.value.length - 1;
                    for (i = tarj.selectionEnd; i < end; i += 1) {
                        if (tarj.value.charAt(i) === '\n') {
                            dest = i;
                            break;
                        }
                    }
                }
                tarj.setSelectionRange(dest, dest);
                e.preventDefault();
                return false;
            }
            else {
                return true;
            }
        }
        else if (ctrl && e.keyCode === 85) { // U
            if (tarj !== null) {
                v = tarj.value;
                if (tarj.nodeName.toUpperCase() === 'TEXTAREA') {
                    ls = findlinestart(v, tarj.selectionStart);
                    tarj.value = v.substr(0, ls) + v.substr(tarj.selectionEnd);
                    tarj.setSelectionRange(ls, ls);
                }
                else {
                    tarj.value = v.substr(tarj.selectionEnd);
                    tarj.setSelectionRange(0, 0);
                }
                e.preventDefault();
                return false;
            }
            else {
                return true;
            }
        }
        else if (ctrl && e.keyCode === 87) { // W
            if (tarj !== null) {
                v = tarj.value;
                c = tarj.selectionStart;
                for (i = c - 1;  i > 0; i -= 1) {
                    ch = v.charAt(i);
                    if (ch === ' ' || ch === '\n' || ch === '\t') {
                        break;
                    }
                }
                s = v.substring(0, i);
                tarj.value = s + v.substr(tarj.selectionEnd);
                tarj.setSelectionRange(i, i);
            }
            else {
                // Perhaps we should allow to use ^W->Close Window outside forms...
            }
            e.preventDefault();
            return false;
        }
    }

    function kpress(e) {
        if (e.keyCode === 17 || e.keyCode === 8) {
            e.stopPropagation();
            e.preventDefault();
            return false;
        }
    }

    function kup(e) {
        e.stopPropagation();
        if (e.keyCode === 17) {
            ctrl = false;
        }
        e.preventDefault();
        return false;
    }

    function focus(e) {
        var t = e.target, tp = null, nv = null;
        if (t !== null && t.nodeName.toUpperCase() === 'INPUT') {
            tp = t.getAttributeNode('type');
            if (tp !== null && tp.nodeValue) {
                nv = tp.nodeValue.toLowerCase();
            }
            // tp === null -> default form input type is 'text'
            if (tp === null || nv === 'text' || nv === 'password') {
                tarj = t;
            }
        }
        else if (t !== null && t.nodeName.toUpperCase() === 'TEXTAREA') {
            tarj = t;
        }
        else {
            tarj = null;
        }
        return true;
    }

    function blur(e) {
        tarj = null; 
        return true;
    }

    document.addEventListener("focus", focus, true);
    document.addEventListener("blur", blur, true);
    document.addEventListener("keydown", kdown, false);
    document.addEventListener("keypress", kpress, false);
    document.addEventListener("keyup", kup, false);
}

init();
